#!/usr/bin/env node
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "fs", "ora", "yargs", "inquirer", "./utils"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var fs = require("fs");
    var ora_1 = require("ora");
    var yargs_1 = require("yargs");
    var inquirer_1 = require("inquirer");
    var utils_1 = require("./utils");
    var reactDocs = require('react-docgen');
    /**
     * @description Parse with react-docgen and return componentName if exists
     */
    function getComponentName(fileContent) {
        try {
            var parsed = reactDocs.parse(fileContent);
            return parsed.displayName || '';
        }
        catch (error) {
            return '';
        }
    }
    /**
     * @description Loop into the dir and find React Components.
     * The research occurs based on file extension.
     * Get the content of the file if the extension is correct, build the component with https://github.com/reactjs/react-docgen to get the componentName
     */
    function getComponentsFromDir(path) {
        var files = fs.readdirSync(path);
        return files.reduce(function (acc, fileName) {
            var filePath = path + "/" + fileName;
            var isDirectory = fs.statSync(filePath).isDirectory();
            if (isDirectory) {
                return acc.concat(getComponentsFromDir(filePath));
            }
            if (utils_1.isAvailableFile(fileName)) {
                var fileContent = fs.readFileSync(filePath, {
                    encoding: 'utf-8',
                });
                var componentName = getComponentName(fileContent);
                if (!componentName) {
                    return acc;
                }
                return acc.concat([
                    {
                        name: componentName,
                        path: filePath,
                        content: fileContent,
                    },
                ]);
            }
            return acc;
        }, []);
    }
    /**
     * @description Loop throuth the list of files and verify if the component name is imported into the file
     * If yes, stop the loop
     * If no, return the component information to be deleted
     */
    function verifyImport(component, listOfComponents) {
        for (var _i = 0, listOfComponents_1 = listOfComponents; _i < listOfComponents_1.length; _i++) {
            var file = listOfComponents_1[_i];
            if (utils_1.isImported(component.name, file.content)) {
                return;
            }
        }
        return component;
    }
    /**
     * @description For each component verify if is imported on the list of files and create a list of unused components
     */
    function getUnusedComponents(components) {
        return components.reduce(function (acc, curr) {
            var componentUnUsed = verifyImport(curr, components);
            if (!componentUnUsed) {
                return acc;
            }
            return acc.concat([componentUnUsed]);
        }, []);
    }
    /**
     * @description Loop all components to delete and ask with a question a confirmation
     */
    function confirmDelete(components) {
        return __awaiter(this, void 0, void 0, function () {
            var _i, components_1, component, answer;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _i = 0, components_1 = components;
                        _a.label = 1;
                    case 1:
                        if (!(_i < components_1.length)) return [3 /*break*/, 4];
                        component = components_1[_i];
                        if (yargs_1.argv.verbose) {
                            console.log(component.content);
                        }
                        return [4 /*yield*/, inquirer_1.prompt([
                                {
                                    type: 'confirm',
                                    name: 'confirm',
                                    message: "Do you want to delete " + component.path + " ?",
                                },
                            ])];
                    case 2:
                        answer = _a.sent();
                        //@ts-ignore
                        if (answer.confirm) {
                            utils_1.deleteComponent(component);
                        }
                        _a.label = 3;
                    case 3:
                        _i++;
                        return [3 /*break*/, 1];
                    case 4: return [2 /*return*/];
                }
            });
        });
    }
    (function () {
        return __awaiter(this, void 0, void 0, function () {
            var spinner, path, components, zombieComponents;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        spinner = ora_1.default({
                            text: 'Searching zombie components',
                        }).start();
                        path = yargs_1.argv.path || process.cwd();
                        components = getComponentsFromDir(path);
                        console.log("\n\n" + components.length + " components found! \n");
                        zombieComponents = getUnusedComponents(components);
                        spinner.stop();
                        console.log(zombieComponents.length + " unused components found! \n");
                        if (yargs_1.argv.force) {
                            return [2 /*return*/, utils_1.deleteComponents(zombieComponents)];
                        }
                        return [4 /*yield*/, confirmDelete(zombieComponents)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/, console.log('\nBye bye!')];
                }
            });
        });
    }());
});
//# sourceMappingURL=index.js.map